/**
 *
 */
package edu.agh.sm2006.nmea2kml;

import java.io.File;
import java.io.FileNotFoundException;
import java.text.ParseException;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Scanner;

import com.drew.imaging.jpeg.JpegMetadataReader;
import com.drew.imaging.jpeg.JpegProcessingException;
import com.drew.metadata.Directory;
import com.drew.metadata.Metadata;
import com.drew.metadata.MetadataException;

/**
 * @author Kuba Madej
 *
 */
public class NMEA2KML {
	private static final String RMC_MESSAGE_INDICATOR = "$GPRMC";

	private static final int EXIF_CREATION_DATE_TAG = 306;

	private final String dataFile;

	private final String photosLocation;

	private final long timeDifference;

	private final PathMode mode;

	private List<Location> locations = new LinkedList<Location>();

	/**
	 * @param dataFile
	 * @param photosLocation
	 * @param timeDifference
	 * @param mode
	 */
	public NMEA2KML(final String dataFile, final String photosLocation,
			String timeDifference, final PathMode mode) {
		this.dataFile = dataFile;
		this.photosLocation = photosLocation;
		this.timeDifference = Long.parseLong(timeDifference);
		this.mode = mode;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		if (args.length != 4) {
			printUsageInfo();
			return;
		}

		final String dataFile = args[0];
		final String photosLocation = args[1];
		final String timeDifference = args[2];
		final PathMode mode;
		try {
			mode = PathMode.valueOf(args[3]);
		} catch (IllegalArgumentException ex) {
			printUsageInfo();
			return;
		}

		new NMEA2KML(dataFile, photosLocation, timeDifference, mode).run();
	}

	private void run() {
		final Scanner scanner;
		try {
			scanner = new Scanner(new File(dataFile));
		} catch (FileNotFoundException e) {
			System.err.println("Data file not found");
			return;
		}

		scanner.useDelimiter("\n");

		while (scanner.hasNext()) {
			final String data = scanner.next();
			if (!data.startsWith(RMC_MESSAGE_INDICATOR))
				continue;
			try {
				locations.add(new Location(data));
			} catch (ParseException e) {
				System.err.println("Couldn't parse data, skipping");
				continue;
			}
		}

		final Printer printer = mode.getPrinter();

		printer.startDocument();

		printer.printLocations(locations);

		File photos = new File(photosLocation);

		File files[] = photos.listFiles();

		if (files != null)
			files: for (File file : files) {
				final String fileName = file.getPath();
				final String fileNameUpperCase = fileName.toUpperCase();
				if (fileNameUpperCase.endsWith(".JPG")
						|| fileNameUpperCase.endsWith(".JPEG")) {
					final Metadata metadata;
					try {
						metadata = JpegMetadataReader.readMetadata(file);
					} catch (JpegProcessingException e) {
						System.err
								.println("Couldn't read EXIF data, image skipped");
						continue;
					}

					final Iterator iterator = metadata.getDirectoryIterator();

					while (iterator.hasNext()) {
						final Directory directory = (Directory) iterator.next();

						final Date creationDate;
						try {
							creationDate = directory
									.getDate(EXIF_CREATION_DATE_TAG);
						} catch (MetadataException e) {
							System.err
									.println("Couldn't read EXIF data, tag directory skipped");
							continue;
						}

						for (Location location : locations) {
							if (location.getTime().getTime() + timeDifference >= creationDate
									.getTime()) {
								printer.printPhoto(location, fileName);
								break;
							}
						}

						continue files;
					}

					System.err
							.println("Couldn't read EXIF data, image skipped");
				}
			}
		else
			System.err
					.println("Couldn't read images directory, images skipped");

		printer.endDocument();
	}

	private static void printUsageInfo() {
		System.out.println("Usage:");
		System.out
				.println("\tjava edu.agh.sm2006.nmea2kml.NMEA2KML <NMEA data file> <photos location> <time difference> <path mode>");
		System.out.println("where <path mode> = plain|color|height");
	}

}
